<?php

class Engine_Package_Archive
{
  static public function inflate($file, $outputPath)
  {
    // Sanity
    if( !file_exists($file) || !is_file($file) || strtolower(pathinfo($file, PATHINFO_EXTENSION)) != 'tar' ) {
      throw new Engine_Package_Exception('File does not exist or is not a tar file');
    }
    if( !file_exists($outputPath) || !is_dir($outputPath) || !is_writeable($outputPath) ) {
      throw new Engine_Package_Exception('Output path does not exist, is not a directory, or is not writeable');
    }
    self::_loadArchiveClass();

    // Make other paths
    $outputSubPath = pathinfo($file, PATHINFO_FILENAME);
    $outputFullPath = $outputPath . DIRECTORY_SEPARATOR . $outputSubPath;

    // If output path already exists, remove
    if( file_exists($outputFullPath) ) {
      self::_rmdirRecursive($outputFullPath, true);
    }

    // Try to make full output path
    if( !is_dir($outputFullPath) ) {
      if( !mkdir($outputFullPath, 0777, true) ) {
        throw new Engine_Package_Exception('Unable to create output folder');
      }
    }

    // Extract
    $archive = new Archive_Tar($file);
    $rval = $archive->extract($outputFullPath);

    // Throw error if failed
    if( $archive->isError($rval) ) {
      throw new Engine_Package_Exception('Error in archive: ' . $rval->getMessage());
    }

    return $outputFullPath;
  }

  static public function deflate(Engine_Package_File $package, $outputPath)
  {
    // Sanity
    if( !file_exists($outputPath) || !is_dir($outputPath) || !is_writeable($outputPath) ) {
      throw new Engine_Package_Exception('Output path does not exist, is not a directory, or is not writeable');
    }
    if( !is_dir($package->getBasePath()) ) {
      throw new Engine_Package_Exception('Missing package base path');
    }
    self::_loadArchiveClass();

    // Make filenames and paths
    $basePath = $package->getBasePath();
    $archiveFile = sprintf('%s-%s-%s.tar', $package->getType(), $package->getName(), $package->getVersion());
    $archiveFullPath = $outputPath . '/' . $archiveFile;

    if( file_exists($archiveFullPath) && !unlink($archiveFullPath) ) {
      throw new Engine_Package_Exception('Target archive already exists and unable to remove');
    }

    // Start packing
    $archive = new Archive_Tar($archiveFullPath);

    // Add main package file
    $rval = $archive->addString('package.json', $package->toString('json'));
    if( $archive->isError($rval) ) {
      throw new Engine_Package_Exception('Error in archive: ' . $rval->getMessage());
    }

    // Add all directories, files, and subpackages
    self::_addPackageToArchive($package, $archive, $basePath);

    return $archiveFullPath;
  }

  static public function readPackageFile($file)
  {
    // Sanity
    if( !file_exists($file) || !is_file($file) || strtolower(pathinfo($file, PATHINFO_EXTENSION)) != 'tar' ) {
      throw new Engine_Package_Exception('File does not exist or is not a tar file');
    }
    self::_loadArchiveClass();

    // Create archive object
    $archive = new Archive_Tar($file);

    // List files
    $fileList = $archive->listContent();
    if( empty($fileList) ) {
      throw new Engine_Package_Exception('Unable to open archive');
    }

    // Check for root package file
    $rootPackageFile = null;
    foreach( $fileList as $arFile ) {
      if( $arFile['filename'] == 'package.json' ) {
        $rootPackageFile = $arFile['filename'];
        break;
      }
    }
    if( null === $rootPackageFile ) {
      throw new Engine_Package_Exception('Root package file not found.');
    }

    // Start building package stuff
    $packageFileObject = new Engine_Package_File();
    $packageFileObject->fromString($archive->extractInString($rootPackageFile), 'json');
    return $packageFileObject;
  }

  static protected function _loadArchiveClass()
  {
    include_once 'Archive/Tar.php';
    if( !class_exists('Archive_Tar', false) ) {
      throw new Engine_Package_Exception('Unable to load Archive_Tar class');
    }
  }

  static protected function _addPackageToArchive(Engine_Package_File $package, Archive_Tar $archive, $basePath)
  {
    $rval = null;
    foreach( $package->getContents() as $name => $contents ) {
      switch( $contents['type'] ) {
        case 'package':
          $subPackageObject = new Engine_Package_File($basePath . '/' . $contents['path'] . '/' . $contents['packageFile']);
          $subPackageObject->setBasePath($basePath);
          self::_addPackageToArchive($subPackageObject, $archive, $basePath);
          break;
        case 'directory':
          // Add directory
          $rval = $archive->addModify($contents['path'], null, $basePath);
          break;
        case 'file':
          // Add file
          $rval = $archive->addModify($contents['path'], null, $basePath);
          break;
        default:
          throw new Engine_Package_Exception('unknown contents type');
          break;
      }
    }
    
    // Throw error if failed
    if( $archive->isError($rval) ) {
      throw new Engine_Package_Exception('Error in archive: ' . $rval->getMessage());
    }
  }

  static protected function _rmdirRecursive($path, $includeSelf = false)
  {
    $it = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($path, RecursiveDirectoryIterator::KEY_AS_PATHNAME), RecursiveIteratorIterator::CHILD_FIRST);
    $unique = array();
    while( $it->valid() ) {
      $key = $it->key();
      if( isset($unique[$key]) ) {
        $it->next();
        continue;
      }
      $unique[$key] = true;

      if( !$it->isDot() ) {
        if( $it->isDir() ) {
          if( !rmdir($key) ) {
            throw new Engine_Package_Exception('Unable to remove directory');
          }
        } else if( $it->isFile() ) {
          if( !unlink($key) ) {
            throw new Engine_Package_Exception('Unable to remove directory');
          }
        }
      }
      
      $it->next();
    }
    
    if( $includeSelf ) {
      if( !rmdir($path) ) {
        throw new Engine_Package_Exception('Unable to remove directory');
      }
    }
  }
}