<?php

class Engine_Package_Diff
{
  // Normal
  const STATE_ADDED = 1;
  const STATE_REMOVED = 2;
  const STATE_CHANGED = 3;
  const STATE_IDENTICAL = 5;

  const STATE_MISSING = 10; // Auto type

  // Conflicts
  const CONFLICT_ADD_EXISTING = 100;
  const CONFLICT_REMOVE_MODIFIED = 101;
  const CONFLICT_CHANGED_BOTH = 102;
  const CONFLICT_KEEP_WORKING = 103; // This is not really a conflict

  // Actions
  const ACTION_ADD = 200;
  const ACTION_REMOVE = 201;
  const ACTION_REPLACE = 202;
  const ACTION_IGNORE = 203;

  // Errors
  const ERROR_BAD_TYPE = 300;
  const ERROR_BOTH_MISSING = 301;
  const ERROR_IMPOSSIBLE = 302;
  const ERROR_UNKNOWN = 303;
  const ERROR_UNKNOWN2 = 304;

  // Indexes
  protected static $_stateTypes = array(
    self::STATE_ADDED,
    self::STATE_REMOVED,
    self::STATE_CHANGED,
    self::STATE_MISSING,
    self::STATE_IDENTICAL,
    self::STATE_MISSING,
  );
  protected static $_conflictTypes = array(
    self::CONFLICT_ADD_EXISTING,
    self::CONFLICT_REMOVE_MODIFIED,
    self::CONFLICT_CHANGED_BOTH,
    self::CONFLICT_KEEP_WORKING,
  );
  protected static $_actionTypes = array(
    self::ACTION_ADD,
    self::ACTION_REMOVE,
    self::ACTION_REPLACE,
    self::ACTION_IGNORE,
  );
  protected static $_errorTypes = array(
    self::ERROR_BAD_TYPE,
    self::ERROR_BOTH_MISSING,
    self::ERROR_IMPOSSIBLE,
    self::ERROR_UNKNOWN,
    self::ERROR_UNKNOWN2,
  );
  protected static $_reverse = array();



  /**
   * Compare a package to a path
   * 
   * @param Engine_Package_File $package The new package
   * @param string $path The (old) main path (working copy ;))
   * @return array
   */
  static public function compare(Engine_Package_File $package, $path)
  {
    $results = self::compareRaw(
      $package->buildFileList($path),
      $path,
      $package->buildFileList(),
      $package->getBasePath()
    );
    return $results;
  }

  /**
   * Three-way compare package to package to path
   * 
   * @param Engine_Package_File $packageOld The old package
   * @param Engine_Package_File $packageNew The new package
   * @param string $path The (old) main path (working copy ;))
   * @return array
   */
  static public function compare3(Engine_Package_File $packageOld, Engine_Package_File $packageNew, $path)
  {
    // This will tell us what was changed by the user
    $resultsOldUser = self::compareRaw(
      $packageOld->buildFileList(),
      $packageOld->getBasePath(),
      $packageOld->buildFileList($path),
      $path
    );

    // This tells us what was changed by the developer
    $resultsOldNew = self::compareRaw(
      $packageOld->buildFileList(),
      $packageOld->getBasePath(),
      $packageNew->buildFileList(),
      $packageNew->getBasePath()
    );

    $results = self::compareRaw3($resultsOldUser, $resultsOldNew);
    
    return $results;
  }

  static public function compareRaw($listOld, $pathOld, $listNew, $pathNew)
  {
    // Build a conglomerate list (by key)
    $combined = array_unique(array_merge(array_keys($listOld), array_keys($listNew)));
    //var_dump($combined);

    // Now diff the paths
    $results = array();
    foreach( $combined as $key ) {
      $results[$key] = self::comparePath($pathOld . '/' . $key, $pathNew . '/' . $key);
    }
    return $results;
  }

  static public function compareRaw3($resultsOldUser, $resultsOldNew)
  {
    // Build a full list of files :O
    $combined = array_unique(array_merge(array_keys($resultsOldUser), array_keys($resultsOldNew)));
    
    // Now diff the paths
    $results = array();
    foreach( $combined as $key ) {
      $results[$key] = self::comparePath3($key, $resultsOldUser[$key], $resultsOldNew[$key]);
    }
    
    return $results;
  }
  
  static public function comparePath($old, $new)
  {
    $old_info = self::_checkCompare($old);
    $new_info = self::_checkCompare($new);
    $status = null;
    $info = null;

    // This should describe the state from old->new
    // Case: $old bad type or $new bad type
    if( null === $old_info || null === $new_info ) {
      $status = self::ERROR_BAD_TYPE;
    }
    // Case: both $old and $new missing (wth?)
    else if( false === $old_info && false === $new_info ) {
      $status = self::ERROR_BOTH_MISSING;
    }
    // Case: $old is missing
    else if( false === $old_info ) {
      $status = self::STATE_MISSING;
      $info = $new_info;
    }
    // Case: $new is missing (the file was removed)
    else if( false === $new_info ) {
      $status = self::STATE_REMOVED;
      $info = $old_info;
    }
    // Case: we are all going insane
    else if( !is_array($old_info) || !is_array($new_info) ) {
      $status = self::ERROR_BAD_TYPE;
    }

    // We can return now if we have a status code
    if( null !== $status ) {
      return array(
        'status' => $status,
        'info' => $info,
        'old' => $old_info,
        'new' => $new_info,
      );
    }

    // Okay, if we got here, we have information on both files
    $diff = array_diff_assoc($old_info, $new_info);
    unset($diff['perms']);

    // Case: nothing changed (yay!)
    if( empty($diff) ) {
      $status = self::STATE_IDENTICAL;
      $info  = $old_info; // Or new-info? ;)
    }
    // Case: hash changed
    else if( !empty($diff['sha1']) ) {
      // We can't start diffing now, because what if this is a three-way diff?
      //$textdiff = self::textDiff($old, $new);
      $status = self::STATE_CHANGED;
      $info = $diff;
    }
    // Case: somthing else changed (wth mk+9000)
    else {
      $status = self::STATE_CHANGED; // Should probably change the status here
      $info = $diff;
    }

    return array(
      'status' => $status,
      'info' => $info,
      'old' => $old_info,
      'new' => $new_info,
    );
  }

  /**
   * Compare two path diff results
   * 
   * Pairings: (old-new, old-working)
   *
   *    ERROR_BAD_TYPE + [ALL] = ERROR_BAD_TYPE
   *    ERROR_BOTH_MISSING + [ALL] = ERROR_BOTH_MISSING
   *    ERROR_IMPOSSIBLE + [ALL] = ERROR_IMPOSSIBLE
   *    ERROR_UNKNOWN + [ALL] = ERROR_UNKNOWN
   *
   *    STATE_ADDED + STATE_ADDED = CONFLICT_ADD_EXISTING || STATE_IDENTICAL
   *      only if the files are not identical
   *    STATE_ADDED + STATE_REMOVED = ACTION_ADD
   *    STATE_ADDED + STATE_CHANGED = ERROR_IMPOSSIBLE
   *      It couldn't have been changed by the user if it didn't exist in the previous version
   *    STATE_ADDED + STATE_IDENTICAL = ERROR_IMPOSSIBLE
   *      Could not be identical as it didn't exist before
   *    STATE_ADDED + STATE_MISSING = ACTION_ADD
   *
   *    STATE_REMOVED + STATE_ADDED = ERROR_IMPOSSIBLE
   *      it can't has been removed from the old package and added by the user
   *    STATE_REMOVED + STATE_REMOVED = ACTION_IGNORE
   *      if it was removed by the user, and is set to be removed, it's the same
   *    STATE_REMOVED + STATE_CHANGED = CONFLICT_REMOVE_MODIFIED
   *    STATE_REMOVED + STATE_IDENTICAL = ACTION_REMOVE
   *      Normal file removal
   *    STATE_REMOVED + STATE_MISSING = ACTION_IGNORE
   *
   *    STATE_CHANGED + STATE_ADDED = ERROR_IMPOSSIBLE
   *      Could not have been changed if added by the user
   *    STATE_CHANGED + STATE_REMOVED = ACTION_ADD
   *      This could be an error, but instead just add it back
   *    STATE_CHANGED + STATE_CHANGED = CONFLICT_CHANGED_BOTH || ACTION_IGNORE
   *      Conflict only if the files are not identical
   *    STATE_CHANGED + STATE_IDENTICAL = ACTION_REPLACE
   *      Normal file update
   *    STATE_CHANGED + STATE_MISSING = ACTION_ADD
   *      Weirdness
   *
   *    STATE_IDENTICAL + STATE_ADDED = ERROR_IMPOSSIBLE
   *      Could not have been added if it existed before
   *    STATE_IDENTICAL + STATE_REMOVED = ACTION_ADD
   *      Add it back
   *    STATE_IDENTICAL + STATE_CHANGED = CONFLICT_KEEP_WORKING || ACTION_IGNORE
   *      Ignore if identical with the user copy
   *    STATE_IDENTICAL + STATE_IDENTICAL = ACTION_IGNORE
   *      Pretty damn straight-forward
   *    STATE_IDENTICAL + STATE_MISSING = ACTION_ADD
   *      Weirdness
   *
   *    STATE_MISSING + [ALL] = ACTION_IGNORE
   * 
   * @param string $path
   * @param array|null $oldResult The result for old-working
   * @param array|null $newResult The result for old-new
   * @return array
   */
  static public function comparePath3($path, $oldResult, $newResult)
  {
    $oldStatus = ( !empty($oldResult['status']) ? $oldResult['status'] : self::STATE_MISSING );
    $newStatus = ( !empty($newResult['status']) ? $newResult['status'] : self::STATE_MISSING );
    $combinedStatus = null;
    
    // Whoops kind of made the func args backwards from the doc block
    $composite = 1000 * $newStatus + $oldStatus;
    switch( $composite ) {
      // Added
      case (self::STATE_ADDED * 1000 + self::STATE_ADDED):
        if( !empty($oldResult['new']['sha1']) && @$oldResult['new']['sha1'] == @$newResult['new']['sha1'] ) {
          $combinedStatus = self::ACTION_IGNORE;
        } else {
          $combinedStatus = self::CONFLICT_ADD_EXISTING;
        }
        break;
      case (self::STATE_ADDED * 1000 + self::STATE_REMOVED):
        $combinedStatus = self::ACTION_ADD;
        break;
      case (self::STATE_ADDED * 1000 + self::STATE_CHANGED):
        $combinedStatus = self::ERROR_IMPOSSIBLE;
        break;
      case (self::STATE_ADDED * 1000 + self::STATE_IDENTICAL):
        $combinedStatus = self::ERROR_IMPOSSIBLE;
        break;
      case (self::STATE_ADDED * 1000 + self::STATE_MISSING):
        $combinedStatus = self::ACTION_ADD;
        break;
      // Removed
      case (self::STATE_REMOVED * 1000 + self::STATE_ADDED):
        $combinedStatus = self::ERROR_IMPOSSIBLE;
        break;
      case (self::STATE_REMOVED * 1000 + self::STATE_REMOVED):
        $combinedStatus = self::ACTION_IGNORE;
        break;
      case (self::STATE_REMOVED * 1000 + self::STATE_CHANGED):
        $combinedStatus = self::CONFLICT_REMOVE_MODIFIED;
        break;
      case (self::STATE_REMOVED * 1000 + self::STATE_IDENTICAL):
        $combinedStatus = self::ACTION_REMOVE;
        break;
      case (self::STATE_REMOVED * 1000 + self::STATE_MISSING):
        $combinedStatus = self::ACTION_IGNORE;
        break;
      // Changed
      case (self::STATE_CHANGED * 1000 + self::STATE_ADDED):
        $combinedStatus = self::ERROR_IMPOSSIBLE;
        break;
      case (self::STATE_CHANGED * 1000 + self::STATE_REMOVED):
        $combinedStatus = self::ACTION_ADD;
        break;
      case (self::STATE_CHANGED * 1000 + self::STATE_CHANGED):
        if( !empty($oldResult['new']['sha1']) && @$oldResult['new']['sha1'] == @$newResult['new']['sha1'] ) {
          $combinedStatus = self::ACTION_IGNORE;
        } else {
          $combinedStatus = self::CONFLICT_CHANGED_BOTH;
        }
        break;
      case (self::STATE_CHANGED * 1000 + self::STATE_IDENTICAL):
        $combinedStatus = self::ACTION_REPLACE;
        break;
      case (self::STATE_CHANGED * 1000 + self::STATE_MISSING):
        $combinedStatus = self::ACTION_ADD;
        break;
      // Identical
      case (self::STATE_IDENTICAL * 1000 + self::STATE_ADDED):
        $combinedStatus = self::ERROR_IMPOSSIBLE;
        break;
      case (self::STATE_IDENTICAL * 1000 + self::STATE_REMOVED):
        $combinedStatus = self::ACTION_ADD;
        break;
      case (self::STATE_IDENTICAL * 1000 + self::STATE_CHANGED):
        if( !empty($oldResult['new']['sha1']) && @$oldResult['new']['sha1'] == @$newResult['new']['sha1'] ) {
          var_dump($oldResult['new']['sha1']);
          var_dump($newResult['new']['sha1']);
          var_dump($oldResult['old']['sha1']);
          var_dump($newResult['old']['sha1']);
          die();
          $combinedStatus = self::ACTION_IGNORE;
        } else {
          $combinedStatus = self::CONFLICT_KEEP_WORKING;
        }
        break;
      case (self::STATE_IDENTICAL * 1000 + self::STATE_IDENTICAL):
        $combinedStatus = self::ACTION_IGNORE;
        break;
      case (self::STATE_IDENTICAL * 1000 + self::STATE_MISSING):
        $combinedStatus = self::ACTION_ADD;
        break;

      // Everything else
      default:
        if( self::getCodeType($newStatus) == 'error' ) {
          $combinedStatus = $newStatus;
        }
        else if( self::getCodeType($oldStatus) == 'error' ) {
          $combinedStatus = $oldStatus;
        }
        else if( $newStatus == self::STATE_MISSING ) {
          $combinedStatus = self::ACTION_IGNORE;
        }
        else {
          // Weirdness
          $combinedStatus = self::ERROR_UNKNOWN2;
        }
        break;
    }

    // Debug
    //if( $combinedStatus == self::ACTION_ADD ) {
    //  var_dump($oldResult);
    //  var_dump($newResult);
    //  die();
    //}

    return array(
      'status' => $combinedStatus,
      'old' => $oldResult,
      'new' => $newResult,
    );
  }
  
  static protected function _checkCompare($arg)
  {
    if( is_string($arg) ) {
      if( file_exists($arg) ) {
        return Engine_Package_File::build_file($arg);
      } else {
        return false;
      }
    } else if( is_array($arg) ) {
      return $arg;
    } else {
      return null;
    }
  }

  static public function textDiff($old, $new)
  {
    include_once "Text/Diff.php";
    $diff = new Text_Diff('auto', array(file($old), file($new)));
    return $diff; // meh
  }

  static public function getCodeType($code)
  {
    if( empty(self::$_reverse) ) {
      self::$_reverse = array();
      self::$_reverse += array_combine(self::$_stateTypes, array_fill(0, count(self::$_stateTypes), 'state'));
      self::$_reverse += array_combine(self::$_conflictTypes, array_fill(0, count(self::$_conflictTypes), 'conflict'));
      self::$_reverse += array_combine(self::$_actionTypes, array_fill(0, count(self::$_actionTypes), 'action'));
      self::$_reverse += array_combine(self::$_errorTypes, array_fill(0, count(self::$_errorTypes), 'error'));
    }
    if( isset(self::$_reverse[$code]) ) {
      return self::$_reverse[$code];
    } else {
      return null;
    }
  }

  static public function getSummaryByCode($results)
  {
    // Create summary by type
    $summary = array();
    foreach( $results as $key => $value ) {
      $code = $value['status'];
      //$type = self::getCodeType($value['status']);
      if( !isset($summary[$code]) ) {
        $summary[$code] = 0;
      }
      $summary[$code]++;
    }
    return $summary;
  }
}