<?php

class Engine_Package_File
{
  protected $_basePath;

  protected $_source;
  
  protected $_structure = array(
    'type' => null,
    'name' => null,
    'version' => null,
    'date' => null,
    'repository' => null,
    'description' => null,
    'callbacks' => null,
    'dependencies' => null,
    'tests' => null,
    'contents' => null,
  );



  // General
  
  public function __construct($source = null, $options = null)
  {
    if( is_string($source) && file_exists($source) ) {
      // Dir mode
      if( is_dir($source) ) {
        $source = $source . DIRECTORY_SEPARATOR . 'package.json'; // @todo someday add xml or php support
      }
      if( is_file($source) ) {
        $this->_setSource($source);
        $this->read($source);
      }
    }
    
    if( is_array($source) ) {
      $this->setOptions($source);
    } else if( is_array($options) ) {
      $this->setOptions($options);
    }
  }

  public function setOptions(array $options)
  {
    foreach( $options as $key => $value ) {
      $method = 'set' . ucfirst($key);
      if( method_exists($this, $method) ) {
        $this->$method($value);
      }
    }

    return $this;
  }

  protected function _setSource($path)
  {
    $this->_source = $path;
  }

  public function getSource()
  {
    if( null === $this->_source ) {
      throw new Engine_Package_Exception('no source file defined');
    }
    return $this->_source;
  }

  public function setBasePath($path)
  {
    $this->_basePath = $path;
    return $this;
  }

  public function getBasePath()
  {
    if( empty($this->_basePath) ) {
      if( defined('APPLICATION_PATH') && is_dir(APPLICATION_PATH) ) {
        $this->_basePath = APPLICATION_PATH;
      } else {
        throw new Engine_Package_Exception('base path cannot be empty');
      }
    }
    return $this->_basePath;
  }



  // Shortcuts

  public function setTests(array $tests)
  {
    $this->addTests($tests);
    return $this;
  }

  public function setDependencies(array $dependencies)
  {
    $this->addDependencies($dependencies);
    return $this;
  }

  public function setFiles(array $paths)
  {
    $this->addFiles($paths);
    return $this;
  }
  
  public function setDirectories(array $paths)
  {
    $this->addDirectories($paths);
    return $this;
  }

  public function setPackages(array $paths)
  {
    $this->addPackages($paths);
    return $this;
  }


  
  // Manipulation

  public function setVersion($version)
  {
    $this->_structure['version'] = (string) $version;
    return $this;
  }
  
  public function getVersion()
  {
    return @$this->_structure['version'];
  }

  public function setName($name)
  {
    $this->_structure['name'] = (string) $name;
    return $this;
  }

  public function getName()
  {
    return @$this->_structure['name'];
  }

  public function setType($type)
  {
    $this->_structure['type'] = (string) $type;
    return $this;
  }

  public function getType()
  {
    return @$this->_structure['type'];
  }

  public function setDate($datetime = null)
  {
    if( null === $datetime ) {
      $datetime = time();
    }
    if( is_string($datetime) ) {
      $datetime = strtotime($datetime);
    }
    if( is_numeric($datetime) ) {
      $this->_structure['date'] = date('r', $datetime);
    }
    return $this;
  }

  public function getDate()
  {
    return @$this->_structure['date'];
  }

  public function setDescription($description)
  {
    $this->_structure['description'] = $description;
    return $this;
  }

  public function getDescription()
  {
    return @$this->_structure['description'];
  }

  public function setRepository($repository)
  {
    $this->_structure['repository'] = $repository;
    return $this;
  }

  public function getRepository()
  {
    return @$this->_structure['repository'];
  }



  // Add

  /**
   * Set a callback
   * 
   * @param string $type The type of callback (sanity, preinstall, install, postinstall)
   * @param string $path The path relative to {self::$_basePath}
   * @param string|array $callback The PHP callback to call (or none if procedural code)
   * @return self
   */
  public function addCallback($type, $path, $callback = null)
  {
    $params = array(
      'type' => $type,
      'path' => $path,
    );

    if( !empty($callback) ) {
      $params['callback'] = $callback;
    }

    $this->_structure['callbacks'][$type] = $params;

    return $this;
  }

  public function addCallbacks(array $callbacks)
  {
    foreach( $callbacks as $key => $value ) {
      $params = array();
      if( !is_numeric($key) ) {
        $params[] = $key;
      }
      if( !is_array($value) ) {
        $params[] = $value;
      } else {
        $params = array_merge($params, $value);
      }
      @list($type, $path, $callback) = @$params;
      $this->addCallback($type, $path, $callback);
    }
  }

  public function addDependency($name, $minVersion = null, $maxVersion = null, $required = true)
  {
    if( null === $required ) { // Blegh
      $required = true;
    }
    $this->_structure['dependencies'][$name] = array(
      'name' => $name,
      'minVersion' => $minVersion,
      'maxVersion' => $maxVersion,
      'required' => (bool) $required,
    );

    return $this;
  }

  public function addDependencies(array $dependencies)
  {
    foreach( $dependencies as $key => $value ) {
      $params = array();
      if( is_string($key) && !is_numeric($key) ) {
        $params[] = $key;
      }
      if( !is_array($value) ) {
        $params[] = $value;
      }
      else
      {
        $params = array_merge($params, $value);
      }
      @list($name, $minVersion, $maxVersion, $required) = @$params;
      $this->addDependency($name, $minVersion, $maxVersion, $required);
    }

    return $this;
  }

  public function addTest($options)
  {
    $key = md5(Zend_Json::encode($options));
    $this->_structure['tests'][$key] = $options;

    return $this;
  }

  public function addTests(array $tests)
  {
    foreach( $tests as $key => $value ) {
      if( !is_numeric($key) ) {
        throw new Engine_Package_Exception('Must only have numeric keys');
      }
      if( !is_array($value) ) {
        throw new Engine_Package_Exception('Invalid test options');
      }
      $this->addTest($value);
    }
    
    return $this;
  }

  public function addFile($path)
  {
    if( !is_file($path) ) {
      throw new Engine_Package_Exception('addFile only takes a file path');
    }
    
    $name = self::sanitize_path($path);
    
    $this->_structure['contents'][$name] = array(
      'type' => 'file',
      'name' => $name,
      'path' => $path,
      'structure' => self::build_file($this->getBasePath() . '/' . $path),
    );

    return $this;
  }

  public function addFiles(array $paths)
  {
    foreach( $paths as $path ) {
      $this->addFile($path);
    }

    return $this;
  }

  public function addDirectory($path)
  {
    if( !is_dir($path) ) {
      throw new Engine_Package_Exception('addDirectory only takes a directory path');
    }

    $name = self::sanitize_path($path);

    $this->_structure['contents'][$name] = array(
      'type' => 'directory',
      'name' => $name,
      'path' => $path,
      'structure' => self::build($this->getBasePath() . '/' . $path),
    );

    return $this;
  }

  public function addDirectories(array $paths)
  {
    foreach( $paths as $path ) {
      $this->addDirectory($path);
    }

    return $this;
  }

  public function addPackage($package)
  {
    if( !is_string($package) ) {
      throw new Engine_Package_Exception('addPackage takes a package filename');
    }
    
    $package = new Engine_Package_File($package);
    try {
      $package->getSource();
    } catch( Exception $e ) {
      throw $e;
      //throw new Engine_Package_Exception('Unable to add package');
    }
    
    $path = $package->getSource();
    $path = str_replace($this->getBasePath() . '/', '', $path);

    // We could use type-name-version or path
    if( $package->getType() && $package->getName() && $package->getVersion() ) {
      $name = self::sanitize_path($package->getType() . '-' . $package->getName() . '-' . $package->getVersion());
    } else {
      $name = self::sanitize_path(dirname($path));
    }

    // Let's also internalize the package data
    $packageData = $package->toArray();
    
    $this->_structure['contents'][$name] = array(
      'type' => 'package',
      'name' => $name,
      'path' => dirname($path),
      'packageFile' => basename($path),
      'packageData' => $packageData,
    );

    return $this;
  }

  public function addPackages(array $paths)
  {
    foreach( $paths as $path ) {
      $this->addPackage($path);
    }

    return $this;
  }



  // Gets

  public function getTests()
  {
    return (array) $this->_structure['tests'];
  }

  public function getContents()
  {
    return (array) @$this->_structure['contents'];
  }


  
  // Read/write
  
  public function read($file)
  {
    switch( pathinfo($file, PATHINFO_EXTENSION) ) {
      case 'json':
        $this->fromString(file_get_contents($file), 'json');
        break;
      case 'xml':
        $this->fromString(simplexml_load_file($file), 'xml');
        break;
      case 'php':
        $this->fromString(include $file, 'php');
        break;
      default:
        throw new Engine_Package_Exception('unknown file format specified');
        break;
    }

    $this->_setSource($file);

    return $this;
  }

  public function write($file)
  {
    $format = pathinfo($file, PATHINFO_EXTENSION);
    if( !in_array($format, array('php', 'json', 'xml')) ) {
      throw new Engine_Package_Exception('unknown file format specified');
    }

    $data = $this->toString($format);

    if( !file_put_contents($file, $data) ) {
      throw new Engine_Package_Exception('Unable to write data to file');
    }

    $this->_setSource($file);

    return $this;
  }

  public function fromString($string, $format = 'json')
  {
    $data = null;

    switch( $format ) {
      case 'json':
        $data = Zend_Json::decode($string);
        break;
      case 'xml':
        $data = self::xml_to_array($string);
        break;
      case 'php':
        $data = $string;
        break;
      default:
        throw new Engine_Package_Exception('unknown format specified');
        break;
    }

    if( empty($data) ) {
      throw new Engine_Package_Exception('Unable to load data');
    }

    $this->_structure = $data;

    return $this;
  }

  public function toString($format = 'json')
  {
    $data = $this->_structure;

    if( empty($data) || !is_array($data) ) {
      throw new Engine_Package_Exception('Unable to write data, no data available');
    }

    switch( $format ) {
      case 'json':
        $data = self::json_format(Zend_Json::encode($data));
        break;
      case 'xml':
        $data = self::xml_format(self::array_to_xml($data));
        break;
      case 'php':
        $data = var_export($data, true);
        break;
      default:
        throw new Engine_Package_Exception('unknown file format specified');
        break;
    }

    if( !is_string($data) ) {
      throw new Engine_Package_Exception('Unable to build data, could not convert to string');
    }

    return $data;
  }

  public function toArray()
  {
    return $this->_structure;
  }
  
  public function buildFileList($basePath = null)
  {
    if( null === $basePath ) {
      $basePath = $this->getBasePath();
    }

    $results = array();
    foreach( $this->getContents() as $name => $contents ) {
      //var_dump($contents);
      switch( $contents['type'] ) {
        case 'package':
          $subpackage = new Engine_Package_File($basePath . '/' . $contents['path'] . '/' . $contents['packageFile'], array(
            'basePath' => $basePath,
          ));
          $results += $subpackage->buildFileList();
          break;
        case 'directory':
          foreach( $contents['structure'] as $subinfo ) {
            $partPath = $contents['path'] . '/' . $subinfo['path'];
            $fullPath = $basePath . '/' . $partPath;
            $results[$partPath] = $fullPath;
          }
          break;
        case 'file':
          $fullPath = $basePath . '/' . $contents['path'];
          $results[$contents['path']] = $fullPath;
          break;
        default:
          throw new Engine_Package_Exception('unknown contents type');
          break;
      }
    }
    return $results;
  }
  


  // Utility

  static public function build($directory)
  {
    $list = array();
    $unique = array();

    // Build
    $it = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($directory, RecursiveDirectoryIterator::KEY_AS_PATHNAME), RecursiveIteratorIterator::SELF_FIRST);
    while( $it->valid() ) {
      // Make sure it's unique
      if( isset($unique[$it->key()]) ) {
        $it->next();
        continue;
      }
      $unique[$it->key()] = true;

      // Add
      $subpath = $it->getSubPathName();

      // Skip dot files, package files and .svn or CVS folders
      if( !$it->isDot() &&
          pathinfo($subpath, PATHINFO_FILENAME) != 'package' &&
          basename($subpath) != '.svn' &&
          basename($subpath) != 'CVS' ) {
        $key = $it->key();
        //$list[$it->getSubPathName()] = array(
        $list[] = array(
          'path' => $it->getSubPathName(),
          'dir' => is_dir($key),
          'file' => is_file($key),
          'perms' => substr(sprintf('%o', $it->getPerms()), -4), // @todo test on windows
          'size' => $it->getSize(),
          'sha1' => sha1_file($key),
        );
      }
      $it->next();
    }

    ksort($list);

    return $list;
  }

  static public function build_file($file)
  {
    if( !file_exists($file) /* || !is_file($file) */ ) {
      throw new Engine_Package_Exception(sprintf('File does not exist: %s', $file));
    }
    return array(
      'path' => basename($file),
      'dir' => is_dir($file),
      'file' => is_file($file),
      'perms' => substr(sprintf('%o', fileperms($file)), -4), // @todo test on windows
      'size' => filesize($file),
      'sha1' => sha1_file($file),
    );
  }

  static public function sanitize_path($path)
  {
    return strtolower(preg_replace('/[^a-zA-Z0-9]+/', '_', $path));
  }
  
  static public function json_format($json)
  {
    $tab = "  ";
    $new_json = "";
    $indent_level = 0;
    $in_string = false;
    $len = strlen($json);

    for($c = 0; $c < $len; $c++)
    {
      $char = $json[$c];
      switch($char)
      {
        case '{':
        case '[':
          if(!$in_string)
          {
            $new_json .= $char . "\n" . str_repeat($tab, $indent_level+1);
            $indent_level++;
          }
          else
          {
            $new_json .= $char;
          }
          break;
        case '}':
        case ']':
          if(!$in_string)
          {
            $indent_level--;
            $new_json .= "\n" . str_repeat($tab, $indent_level) . $char;
          }
          else
          {
            $new_json .= $char;
          }
          break;
        case ',':
          if(!$in_string)
          {
            $new_json .= ",\n" . str_repeat($tab, $indent_level);
          }
          else
          {
            $new_json .= $char;
          }
          break;
        case ':':
          if(!$in_string)
          {
            $new_json .= ": ";
          }
          else
          {
            $new_json .= $char;
          }
          break;
        case '"':
          if($c > 0 && $json[$c-1] != '\\')
          {
            $in_string = !$in_string;
          }
        default:
          $new_json .= $char;
          break;
      }
    }

    return $new_json;
  }
  
  static public function xml_format($xml)
  {
    $dom = new DOMDocument('1.0');
    $dom->loadXML($xml);
    $dom->formatOutput = true;
    return $dom->saveXML();
  }

  static public function xml_to_array($obj)
  {
    if( is_string($obj) ) {
      $obj = new SimpleXMLElement($obj);
    }
    if( !$obj instanceof SimpleXMLElement ) {
      throw new Exception('Must be string or SimpleXMLElement');
    }

    $arr = array();
    $children = $obj->children();
    foreach( $children as $elementName => $node )
    {
      $content = self::xmlToArray($node);
      if( empty($content) ) {
        $content = (string) $node; // @todo check if this works when __toString() not added to PHP yet
      }

      if( $elementName === 'numericNode' ) {
        $arr[] = $content;
      } else {
        $arr[$elementName] = $content;
      }
    }

    return $arr;
  }

  static public function array_to_xml($arr, SimpleXMLElement $xml = null)
  {
    if( null === $xml ) {
      $xml = simplexml_load_string("<?xml version='1.0' encoding='utf-8'?><package />");
    }
    $i = 0;
    foreach( $arr as $key => $value ) {
      // Make name
      $childName = null;
      if( is_numeric($key) ) {
        $childName = 'numericNode'; //sprintf('numeric_node_%d', $key);
      } else {
        $childName = $key;
      }

      // Make value
      $childValue = null;
      if( is_array($value) ) {
        // This is a special case
        $childObject = $xml->addChild($childName);
        $childObject = self::arrayToXml($value, $childObject);
        continue;
      } else if( is_scalar($value) ) {
        if( is_bool($value) ) {
          $childValue = sprintf('%d', $value);
        } else {
          $childValue = $value;
        }
      } else {
        throw new Exception('Unknown data type passed to Engine_Package_File::arrayToXml() - ' . gettype($value));
      }

      // Add child
      $xml->addChild($childName, $childValue);
    }
    return $xml;
  }
}