<?php

class Engine_Package_Manager
{
  const PATH_BASE = 'base';
  const PATH_TEMP = 'temporary';
  const PATH_ARCHIVES = 'archives';
  const PATH_PACKAGES = 'packages';
  const PATH_REPOSITORIES = 'repositories';
  const PATH_INSTALLED = 'installed';
  const PATH_SETTINGS_REPOSITORIES = 'srepositories';

  protected $_basePath;

  protected $_paths = array(
    self::PATH_BASE => null,
    self::PATH_TEMP => 'temporary',
    self::PATH_ARCHIVES => 'temporary/archives',
    self::PATH_PACKAGES => 'temporary/packages',
    self::PATH_REPOSITORIES => 'temporary/repositories',
    self::PATH_INSTALLED => 'application/packages',
    self::PATH_SETTINGS_REPOSITORIES => 'application/settings/repositories.php',
  );
  
  protected $_ftp;

  protected $_repositories;



  // General
  
  public function __construct($options = null)
  {
    if( is_array($options) ) {
      $this->setOptions($options);
    }
  }

  public function setOptions(array $options)
  {
    foreach( $options as $key => $value ) {
      $method = 'set' . ucfirst($key);
      if( method_exists($this, $method) ) {
        $this->$method($value);
      }
    }

    return $this;
  }



  // Paths

  public function setPath($path, $type = self::PATH_BASE) {
    if( !isset($this->_paths[$type]) ) {
      throw new Engine_Package_Manager_Exception('Invalid path type');
    }
    $this->_paths[$type] = $path;
    if( $type == self::PATH_BASE ) {
      $this->_basePath = $path; // B/c
    }
    return $this;
  }

  public function setPaths(array $paths)
  {
    foreach( $paths as $type => $path ) {
      $this->setPath($path, $type);
    }
    return $this;
  }

  public function getAbsPath($type)
  {
    $path = $this->getPath($type);
    if( $type !== self::PATH_BASE && @$path[0] != DIRECTORY_SEPARATOR && @$path[1] != ':' ) {
      $path = $this->_basePath . DS . $path;
    }
    return $path;
  }

  public function getPath($type = self::PATH_BASE)
  {
    if( !isset($this->_paths[$type]) ) {
      throw new Engine_Package_Manager_Exception('Invalid path type');
    }
    if( $type == self::PATH_BASE && null === $this->_paths[$type] ) {
      $this->_paths[$type] = $this->_basePath = APPLICATION_PATH; // B/c
    }
    return $this->_paths[$type];
  }

  public function getPaths()
  {
    return $this->_paths;
  }

  public function setBasePath($path)
  {
    return $this->setPath($path, self::PATH_BASE);
  }

  public function getBasePath()
  {
    return $this->getPath(self::PATH_BASE);
  }



  // FTP
  
  public function setFtp(Net_FTP $ftp)
  {
    $this->_ftp = $ftp;
    return $this;
  }

  public function getFtp()
  {
    return $this->_ftp;
  }



  // Repositories

  public function setRepositories(array $repositories)
  {
    foreach( $repositories as $repository ) {
      $this->_repositories[$repository] = new Engine_Package_Repository(array(
        'uri' => $repository,
        'basePath' => $this->getBasePath(),
      ));
    }

    return $this;
  }

  public function getRepositories()
  {
    if( null === $this->_repositories ) {
      $configFile = $this->getPath(self::PATH_SETTINGS_REPOSITORIES);
      $config = include $configFile;
      if( empty($config) || !is_array($config) ) {
        $this->_repositories = array();
      } else {
        $this->setRepositories($config);
      }
    }

    return $this->_repositories;
  }



  // Actions

  public function apply(array $packages)
  {
    // Check each package
    $packageInfo = array();
    foreach( $packages as $package ) {
      $package = str_replace(':', '-', $package);
      $packageInfo[$package] = array(
        'key' => $package,
        'file' => $this->getPath(self::PATH_ARCHIVES) . DS . $package . '.tar',
        'dir' => $this->getPath(self::PATH_PACKAGES) . DS . $package,
      );
      if( !is_file($packageInfo[$package]['file']) ) {
        throw new Engine_Package_Manager_Exception('Install archive missing');
      }
      if( !is_dir($packageInfo[$package]['dir']) ) {
        // Try to extract
        try {
          Engine_Package_Archive::inflate($packageInfo[$package]['file'], dirname($packageInfo[$package]['dir']));
        } catch( Exception $e ) {
          Engine_Package_Utilities::fsRmdirRecursive($packageInfo[$package]['dir'], true);
          throw $e;
        }
      }
      $packageInfo[$package]['package'] = new Engine_Package_File($packageInfo[$package]['dir']);
    }


    // Copy all to target
    $ftp = $this->getFtp();

    foreach( $packageInfo as $info ) {
      $packageFile = $info['package'];
      //$files = $packageFile->buildFileList();
      foreach( $packageFile->getContents() as $content ) {
        $targetPath = $this->getBasePath() . '/' . $content['path'];
        $sourcePath = $info['dir'] . '/' . $content['path'];
        switch( $content['type'] ) {
          case 'package':
          case 'directory':

            // The good stuff
            if( null === $ftp ) {
              Engine_Package_Utilities::fsCopyRecursive($sourcePath, $targetPath);
            } else {
              $ret = $ftp->putRecursive($sourcePath, $targetPath, true);
              if( PEAR::isError($ret) ) {
                throw new Engine_Package_Manager_Exception('Unable to put');
              }
            }

            break;
          case 'file':

            // The good stuff
            if( null === $ftp ) {
              if( !copy($sourcePath, $targetPath) ) {
                throw new Engine_Package_Manager_Exception('Unable to copy');
              }
            } else {
              $ret = $ftp->put($sourcePath, $targetPath, true);
              if( PEAR::isError($ret) ) {
                throw new Engine_Package_Manager_Exception('Unable to put');
              }
            }
            
            break;
          default:
            throw new Engine_Package_Manager_Exception('Unknown contents type');
            break;
        }
      }

      // And lastly, put the main package file into the installed packages folder
      $ippath = $this->getPath(self::PATH_INSTALLED) . DS . $info['key'] . '.json';
      $sppath = $packageFile->getSource();
      if( null === $ftp ) {
        if( !copy($sppath, $ippath) ) {
          throw new Engine_Package_Manager_Exception('Unable to copy');
        }
      } else {
        $ret = $ftp->put($sppath, $ippath, true);
        if( PEAR::isError($ret) ) {
          throw new Engine_Package_Manager_Exception('Unable to put');
        }
      }
    }

    
  }




  // Informational
  
  public function listInstalledPackages()
  {
    $installedPackages = array();
    $it = new DirectoryIterator($this->getPath(self::PATH_INSTALLED));

    // List installed packages
    foreach( $it as $file ) {
      if( $file->isDot() || $file->isDir() ) continue;
      try {
        $packageFile = new Engine_Package_File($file->getPathname());
        // Reset base path
        $packageFile->setBasePath($this->_basePath);
        $installedPackages[] = $packageFile;
      } catch( Exception $e ) {
        // Silence?
        if( APPLICATION_ENV == 'development' ) {
          throw $e;
        }
      }
    }

    return $installedPackages;
  }

  public function listAvailablePackages()
  {
    $availablePackages = array();
    $it = new DirectoryIterator($this->getPath(self::PATH_ARCHIVES));
    foreach( $it as $file ) {
      if( $file->isDot() || $file->isDir() ) continue;
      try {
        $packageFile = Engine_Package_Archive::readPackageFile($file->getPathname());
        $availablePackages[] = $packageFile;
      } catch( Exception $e ) {
        // Silence?
        if( APPLICATION_ENV == 'development' ) {
          throw $e;
        }
      }
    }

    return $availablePackages;
  }

  public function listExtractedPackages()
  {
    $extractedPackages = array();
    $it = new DirectoryIterator($this->getPath(self::PATH_PACKAGES));
    foreach( $it as $file ) {
      if( $file->isDot() || $file->isDir() ) continue;
      try {
        $packageFile = new Engine_Package_File($file->getPathname());
        $extractedPackages[] = $packageFile;
      } catch( Exception $e ) {
        // Silence?
        if( APPLICATION_ENV == 'development' ) {
          throw $e;
        }
      }
    }

    return $extractedPackages;
  }

  public function resolveDependencies($packages)
  {
    if( !is_array($packages) ) {
      $packages = array($packages);
    }
    foreach( $packages as $package ) {
      if( !($package instanceof Engine_Package_File) ) {
        throw new Engine_Package_Manager_Exception('Not a valid package');
      }
    }
  }
}