<?php

class Engine_Package_Repository
{
  protected $_uri;

  protected $_client;

  protected $_basePath;

  protected $_temporaryPath = 'application/temporary/package';

  protected $_cachePath = 'application/temporary/repositories';

  protected $_throwExceptions = false;

  protected $_cacheLifetime = 86400;
  
  public function __construct(array $options = null)
  {
    if( is_array($options) ) {
      $this->setOptions($options);
    }

    if( null === $this->_uri ) {
      throw new Engine_Package_Exception('No uri supplied');
    }

    if( null === $this->_basePath ) {
      $this->_basePath = APPLICATION_PATH;
    }
  }

  public function setOptions(array $options)
  {
    foreach( $options as $key => $value ) {
      $method = 'set' . ucfirst($key);
      if( method_exists($this, $method) ) {
        $this->$method($value);
      }
    }
  }

  public function setUri($uri)
  {
    if( !parse_url($uri) ) {
      throw new Engine_Package_Exception('Invalid uri supplied');
    }

    $this->_uri = $uri;
    $this->getClient()->setUri($uri);
    return $this;
  }

  public function getUri()
  {
    if( null === $this->_uri ) {
      throw new Engine_Package_Exception('No uri supplied');
    }
    return $this->_uri;
  }

  public function setBasePath($path)
  {
    if( !is_dir($path) ) {
      throw new Engine_Package_Exception('Base path is not valid');
    }
    $this->_basePath = $path;
    return $this;
  }

  public function getBasePath()
  {
    if( null === $this->_basePath ) {
      throw new Engine_Package_Exception('No base path supplied');
    }
    return $this->_basePath;
  }

  public function setTemporaryPath($path)
  {
    $this->_temporaryPath = $path;
  }

  public function getTemporaryPath()
  {
    if( null === $this->_temporaryPath ) {
      throw new Engine_Package_Exception('No temporary path supplied');
    }
    return $this->_temporaryPath;
  }

  public function setCachePath($path)
  {
    $this->_cachePath = $path;
  }

  public function getCachePath()
  {
    if( null === $this->_cachePath ) {
      throw new Engine_Package_Exception('No cache path supplied');
    }
    return $this->_cachePath;
  }

  public function getClient()
  {
    if( null === $this->_client ) {
      $this->_client = new Zend_Http_Client();
      $this->_client->setCookieJar(true);
      $this->_client->setAdapter('Zend_Http_Client_Adapter_Curl');
    }

    return $this->_client;
  }



  // High level commands

  public function get()
  {
    $cachePath = $this->_basePath . DS . $this->_cachePath;
    $cacheFile = $cachePath . DS . base64_encode($this->_uri . '_' . 'list');
    $data = null;

    // Check cache
    if( file_exists($cacheFile) ) {
      $data = file_get_contents($cacheFile);
      $data = Zend_Json::decode($data);
      // Remove modified structure
      if( empty($data['data']) ) {
        $data = null;
      } else {
        $data = $data['data'];
      }
    }

    // Try to reload
    if( !$data || !is_array($data) || empty($data['data']) || (!empty($data['expires']) && time() > $data['expires'] + $this->_cacheLifetime) ) {
      $data = $this->reload();
    }

    // Whoops
    if( !$data || !is_array($data) ) {
      throw new Engine_Package_Exception('Unable to load package list from repository.');
    }

    return $data;
  }

  public function reload()
  {
    $cachePath = $this->_basePath . DS . $this->_cachePath;
    $cacheFile = $cachePath . DS . base64_encode($this->_uri . '_' . 'list');

    // Load data
    $data = $this->queryList();
    if( !$data || !is_array($data) ) {
      throw new Engine_Package_Exception('Unable to load package list from repository.');
    }

    // Modify data structure for cache
    $cdata = array(
      'expires' => time() + $this->_cacheLifetime,
      'data' => $data,
    );
    
    // Write data to cache (don't bother to except unless it already exists)
    if( !@file_put_contents($cacheFile, Zend_Json::encode($cdata)) && file_exists($cacheFile) ) {
      throw new Engine_Package_Exception('Unable to save package list to file when file already exists.');
    }

    return $data;
  }
  
  public function pull($package, $version = null, $dir = null)
  {
    // Get package info
    // @todo version
    $packageInfo = $this->queryInfo($package);

    // See if requested version is available (or use default version)
    if( null === $version ) {
      $version = $packageInfo['responseData']['package']['version'];
    } else {
      $found = false;
      foreach( $packageInfo['responseData']['versions'] as $versionInfo ) {
        if( $versionInfo['version'] == $version ) {
          $found = true;
        }
      }
      if( !$found ) {
        throw new Engine_Package_Exception('Requested version not available.');
      }
    }
    
    // Check paths
    $packageFile =
      $packageInfo['responseData']['package']['type'] . '-' .
      $packageInfo['responseData']['package']['name'] . '-' .
      $version . '.tar';
    if( null === $dir ) {
      $archiveDir = APPLICATION_PATH . '/temporary/package/archives';
    } else {
      $archiveDir = rtrim($dir, '/');
    }
    $archivePath = $archiveDir . '/' . $packageFile;
    if( !is_dir($archiveDir) ) {
      throw new Engine_Package_Exception('Archive path does not exist');
    }
    if( file_exists($archivePath) ) {
      throw new Engine_Package_Exception('Archive already downloaded');
    }
    
    $packageHttpResponse = $this->queryGet($package, $version);
    if( !($packageHttpResponse instanceof Zend_Http_Response) ) {
      throw new Engine_Package_Exception('Invalid response');
    }
    $packageBody = $packageHttpResponse->getBody();
    if( !file_put_contents($archivePath, $packageBody) ) {
      throw new Engine_Package_Exception('Unable to store archive in file');
    }

    return $archivePath;
  }





  // Commands

  public function queryList()
  {
    return $this->_query('list');
  }

  public function querySearch($string)
  {
    return $this->_query('search', array(
      'query' => $string,
    ));
  }

  public function queryGet($package, $version)
  {
    $response = $this->_query('get', array(
      'package' => $package,
      'version' => $version,
    ), true);

    // Check to make sure it's not an error
    $responseBody = Zend_Json::decode($response->getBody());
    if( !is_string($responseBody) ) {
      $this->_validateResponse($responseBody);
    }
    
    return $response;
  }

  public function queryInfo($package)
  {
    return $this->_query('info', array(
      'package' => $package,
    ));
  }
  


  // Utility

  protected function _query($action, $args = array(), $rawResponse = false)
  {
    $client = $this->getClient();
    //$client = new Zend_Http_Client();

    // Looks like it won't let us pass action as a get param
    $uri = rtrim($this->getUri(), '/') . '/' . $action;

    $client
      ->resetParameters()
      ->setUri($uri)
      ->setParameterGet(array_merge($args, array(
        //'action' => $action,
      )))
      ;

    $response = $client->request();
    
    // Decode response json automatically
    if( !$rawResponse ) {
      $responseBody = Zend_Json::decode($response->getBody());
      $this->_validateResponse($responseBody);
      return $responseBody;
    }

    // Just return the response object
    else
    {
      return $response;
    }
  }

  protected function _validateResponse($responseBody)
  {
    if( empty($responseBody) ) {
      throw new Engine_Package_Exception('no data returned');
    }
    if( is_string($responseBody) ) {
      $responseBody = Zend_Json::decode($responseBody);
    }
    if( !is_array($responseBody) || empty($responseBody) || empty($responseBody['responseStatus']) ) {
      throw new Engine_Package_Exception('invalid data returned');
    }
    if( $responseBody['responseStatus'] !== 200 ) {
      throw new Engine_Package_Exception('repository returned error code: ' . $decodedResponse['responseStatus'], (int) $decodedResponse['responseStatus']);
    }
  }
}